@@ -3,7 +3,7 @@ package DBIx::Class::Schema::Loader::Base;
 use strict;
 use warnings;
 use base qw/Class::Accessor::Grouped Class::C3::Componentised/;
-use Class::C3;
+use mro 'c3';
 use Carp::Clan qw/^DBIx::Class/;
 use DBIx::Class::Schema::Loader::RelBuilder;
 use Data::Dump qw/ dump /;
@@ -18,13 +18,13 @@ use Class::Unload;
 use Class::Inspector ();
 use Scalar::Util 'looks_like_number';
 use File::Slurp 'slurp';
-use DBIx::Class::Schema::Loader::Utils qw/split_name dumper_squashed/;
+use DBIx::Class::Schema::Loader::Utils qw/split_name dumper_squashed eval_without_redefine_warnings/;
 use DBIx::Class::Schema::Loader::Optional::Dependencies ();
 use Try::Tiny;
 use DBIx::Class ();
 use namespace::clean;
 
-our $VERSION = '0.07001';
+our $VERSION = '0.07002';
 
 __PACKAGE__->mk_group_ro_accessors('simple', qw/
                                 schema
@@ -52,7 +52,7 @@ __PACKAGE__->mk_group_ro_accessors('simple', qw/
                                 schema_base_class
                                 result_base_class
                                 use_moose
-				overwrite_modifications
+                                overwrite_modifications
 
                                 relationship_attrs
 
@@ -521,6 +521,11 @@ sub new {
 
     my $self = { %args };
 
+    # don't lose undef options
+    for (values %$self) {
+        $_ = 0 unless defined $_;
+    }
+
     bless $self => $class;
 
     if (my $config_file = $self->config_file) {
@@ -546,8 +551,7 @@ sub new {
 
     if ($self->use_moose) {
         if (not DBIx::Class::Schema::Loader::Optional::Dependencies->req_ok_for('use_moose')) {
-            die sprintf "You must install the following CPAN modules to enable the use_moose option: %s.\nYou are missing: %s.\n",
-                "Moose, MooseX::NonMoose and namespace::autoclean",
+            die sprintf "You must install the following CPAN modules to enable the use_moose option: %s.\n",
                 DBIx::Class::Schema::Loader::Optional::Dependencies->req_missing_for('use_moose');
         }
     }
@@ -650,22 +654,21 @@ EOF
     my $filename = $self->_get_dump_filename($self->schema_class);
     return unless -e $filename;
 
-    open(my $fh, '<', $filename)
-        or croak "Cannot open '$filename' for reading: $!";
+    my ($old_gen, $old_md5, $old_ver, $old_ts, $old_custom) =
+      $self->_parse_generated_file($filename);
 
-    my $load_classes     = 0;
-    my $result_namespace = '';
+    return unless $old_ver;
 
-    while (<$fh>) {
-        if (/^__PACKAGE__->load_classes;/) {
-            $load_classes = 1;
-        } elsif (/result_namespace => '([^']+)'/) {
-            $result_namespace = $1;
-        } elsif (my ($real_ver) =
-                /^# Created by DBIx::Class::Schema::Loader v(\d+\.\d+)/) {
+    # determine if the existing schema was dumped with use_moose => 1
+    if (! defined $self->use_moose) {
+        $self->{use_moose} = 1 if $old_gen =~ /^ (?!\s*\#) use \s+ Moose/xm;
+    }
 
-            if ($load_classes && (not defined $self->use_namespaces)) {
-                warn <<"EOF"  unless $ENV{SCHEMA_LOADER_BACKCOMPAT};
+    my $load_classes = ($old_gen =~ /^__PACKAGE__->load_classes;/m) ? 1 : 0;
+    my $result_namespace = do { ($old_gen =~ /result_namespace => '([^']+)'/) ? $1 : '' };
+
+    if ($load_classes && (not defined $self->use_namespaces)) {
+        warn <<"EOF"  unless $ENV{SCHEMA_LOADER_BACKCOMPAT};
 
 'load_classes;' static schema detected, turning off 'use_namespaces'.
 
@@ -675,39 +678,37 @@ variable to disable this warning.
 See perldoc DBIx::Class::Schema::Loader::Manual::UpgradingFromV4 for more
 details.
 EOF
-                $self->use_namespaces(0);
-            }
-            elsif ($load_classes && $self->use_namespaces) {
-                $self->_upgrading_from_load_classes(1);
-            }
-            elsif ((not $load_classes) && defined $self->use_namespaces
-                                       && (not $self->use_namespaces)) {
-                $self->_downgrading_to_load_classes(
-                    $result_namespace || 'Result'
-                );
-            }
-            elsif ((not defined $self->use_namespaces)
-                   || $self->use_namespaces) {
-                if (not $self->result_namespace) {
-                    $self->result_namespace($result_namespace || 'Result');
-                }
-                elsif ($result_namespace ne $self->result_namespace) {
-                    $self->_rewriting_result_namespace(
-                        $result_namespace || 'Result'
-                    );
-                }
-            }
+        $self->use_namespaces(0);
+    }
+    elsif ($load_classes && $self->use_namespaces) {
+        $self->_upgrading_from_load_classes(1);
+    }
+    elsif ((not $load_classes) && defined $self->use_namespaces && ! $self->use_namespaces) {
+        $self->_downgrading_to_load_classes(
+            $result_namespace || 'Result'
+        );
+    }
+    elsif ((not defined $self->use_namespaces) || $self->use_namespaces) {
+        if (not $self->result_namespace) {
+            $self->result_namespace($result_namespace || 'Result');
+        }
+        elsif ($result_namespace ne $self->result_namespace) {
+            $self->_rewriting_result_namespace(
+                $result_namespace || 'Result'
+            );
+        }
+    }
 
-            # XXX when we go past .0 this will need fixing
-            my ($v) = $real_ver =~ /([1-9])/;
-            $v = "v$v";
+    # XXX when we go past .0 this will need fixing
+    my ($v) = $old_ver =~ /([1-9])/;
+    $v = "v$v";
 
-            last if $v eq $CURRENT_V || $real_ver =~ /^0\.\d\d999/;
+    return if ($v eq $CURRENT_V || $old_ver =~ /^0\.\d\d999/);
 
-            if (not %{ $self->naming }) {
-                warn <<"EOF" unless $ENV{SCHEMA_LOADER_BACKCOMPAT};
+    if (not %{ $self->naming }) {
+        warn <<"EOF" unless $ENV{SCHEMA_LOADER_BACKCOMPAT};
 
-Version $real_ver static schema detected, turning on backcompat mode.
+Version $old_ver static schema detected, turning on backcompat mode.
 
 Set the 'naming' attribute or the SCHEMA_LOADER_BACKCOMPAT environment variable
 to disable this warning.
@@ -717,28 +718,22 @@ See: 'naming' in perldoc DBIx::Class::Schema::Loader::Base .
 See perldoc DBIx::Class::Schema::Loader::Manual::UpgradingFromV4 if upgrading
 from version 0.04006.
 EOF
-            }
-            else {
-                $self->_upgrading_from($v);
-                last;
-            }
 
-            $self->naming->{relationships}    ||= $v;
-            $self->naming->{monikers}         ||= $v;
-            $self->naming->{column_accessors} ||= $v;
+        $self->naming->{relationships}    ||= $v;
+        $self->naming->{monikers}         ||= $v;
+        $self->naming->{column_accessors} ||= $v;
 
-            $self->schema_version_to_dump($real_ver);
-
-            last;
-        }
+        $self->schema_version_to_dump($old_ver);
+    }
+    else {
+        $self->_upgrading_from($v);
     }
-    close $fh;
 }
 
 sub _validate_class_args {
     my $self = shift;
     my $args = shift;
-    
+
     foreach my $k (@CLASS_ARGS) {
         next unless $self->$k;
 
@@ -849,14 +844,7 @@ sub _load_external {
         $code = $self->_rewrite_old_classnames($code);
 
         if ($self->dynamic) { # load the class too
-            # kill redefined warnings
-            my $warn_handler = $SIG{__WARN__} || sub { warn @_ };
-            local $SIG{__WARN__} = sub {
-                $warn_handler->(@_)
-                    unless $_[0] =~ /^Subroutine \S+ redefined/;
-            };
-            eval $code;
-            die $@ if $@;
+            eval_without_redefine_warnings($code);
         }
 
         $self->_ext_stmt($class,
@@ -897,14 +885,7 @@ been used by an older version of the Loader.
 * PLEASE RENAME THIS CLASS: from '$old_class' to '$class', as that is the
 new name of the Result.
 EOF
-            # kill redefined warnings
-            my $warn_handler = $SIG{__WARN__} || sub { warn @_ };
-            local $SIG{__WARN__} = sub {
-                $warn_handler->(@_)
-                    unless $_[0] =~ /^Subroutine \S+ redefined/;
-            };
-            eval $code;
-            die $@ if $@;
+            eval_without_redefine_warnings($code);
         }
 
         chomp $code;
@@ -973,46 +954,26 @@ sub rescan {
 }
 
 sub _relbuilder {
-    no warnings 'uninitialized';
     my ($self) = @_;
 
     return if $self->{skip_relationships};
 
-    if ($self->naming->{relationships} eq 'v4') {
-        require DBIx::Class::Schema::Loader::RelBuilder::Compat::v0_040;
-        return $self->{relbuilder} ||=
-            DBIx::Class::Schema::Loader::RelBuilder::Compat::v0_040->new(
-                $self->schema,
-                $self->inflect_plural,
-                $self->inflect_singular,
-                $self->relationship_attrs,
-            );
-    }
-    elsif ($self->naming->{relationships} eq 'v5') {
-        require DBIx::Class::Schema::Loader::RelBuilder::Compat::v0_05;
-        return $self->{relbuilder} ||= DBIx::Class::Schema::Loader::RelBuilder::Compat::v0_05->new (
-             $self->schema,
-             $self->inflect_plural,
-             $self->inflect_singular,
-             $self->relationship_attrs,
-        );
-    }
-    elsif ($self->naming->{relationships} eq 'v6') {
-        require DBIx::Class::Schema::Loader::RelBuilder::Compat::v0_06;
-        return $self->{relbuilder} ||= DBIx::Class::Schema::Loader::RelBuilder::Compat::v0_06->new (
-             $self->schema,
-             $self->inflect_plural,
-             $self->inflect_singular,
-             $self->relationship_attrs,
-        );
-    }
+    return $self->{relbuilder} ||= do {
 
-    return $self->{relbuilder} ||= DBIx::Class::Schema::Loader::RelBuilder->new (
-             $self->schema,
-             $self->inflect_plural,
-             $self->inflect_singular,
-             $self->relationship_attrs,
-    );
+        no warnings 'uninitialized';
+        my $relbuilder_suff =
+            {qw{
+                v4  ::Compat::v0_040
+                v5  ::Compat::v0_05
+                v6  ::Compat::v0_06
+            }}
+            ->{ $self->naming->{relationships}};
+
+        my $relbuilder_class = 'DBIx::Class::Schema::Loader::RelBuilder'.$relbuilder_suff;
+        eval "require $relbuilder_class"; die $@ if $@;
+        $relbuilder_class->new( $self );
+
+    };
 }
 
 sub _load_tables {
@@ -1101,11 +1062,11 @@ sub _reload_classes {
         
         {
             no warnings 'redefine';
-            local *Class::C3::reinitialize = sub {};
+            local *Class::C3::reinitialize = sub {};  # to speed things up, reinitialized below
             use warnings;
 
-            if ($class->can('meta') && try { $class->meta->isa('Moose::Meta::Class') }) {
-                $class->meta->make_mutable;
+            if (my $mc = $self->_moose_metaclass($class)) {
+                $mc->make_mutable;
             }
             Class::Unload->unload($class) if $unload;
             my ($source, $resultset_class);
@@ -1115,8 +1076,8 @@ sub _reload_classes {
                 && ($resultset_class ne 'DBIx::Class::ResultSet')
             ) {
                 my $has_file = Class::Inspector->loaded_filename($resultset_class);
-                if ($resultset_class->can('meta') && try { $resultset_class->meta->isa('Moose::Meta::Class') }) {
-                    $resultset_class->meta->make_mutable;
+                if (my $mc = $self->_moose_metaclass($resultset_class)) {
+                    $mc->make_mutable;
                 }
                 Class::Unload->unload($resultset_class) if $unload;
                 $self->_reload_class($resultset_class) if $has_file;
@@ -1132,6 +1093,15 @@ sub _reload_classes {
     }
 }
 
+sub _moose_metaclass {
+  return undef unless $INC{'Class/MOP.pm'};   # if CMOP is not loaded the class could not have loaded in the 1st place
+
+  my $mc = Class::MOP::class_of($_[1])
+    or return undef;
+
+  return $mc->isa('Moose::Meta::Class') ? $mc : undef;
+}
+
 # We use this instead of ensure_class_loaded when there are package symbols we
 # want to preserve.
 sub _reload_class {
@@ -1141,12 +1111,9 @@ sub _reload_class {
     delete $INC{ $class_path };
 
 # kill redefined warnings
-    my $warn_handler = $SIG{__WARN__} || sub { warn @_ };
-    local $SIG{__WARN__} = sub {
-        $warn_handler->(@_)
-            unless $_[0] =~ /^Subroutine \S+ redefined/;
+    eval {
+        eval_without_redefine_warnings ("require $class");
     };
-    eval "require $class;";
     die "Failed to reload class $class: $@" if $@;
 }
 
@@ -1285,44 +1252,55 @@ sub _write_classfile {
         warn "Deleting existing file '$filename' due to "
             . "'really_erase_my_files' setting\n" unless $self->{quiet};
         unlink($filename);
-    }    
+    }
 
-    my ($custom_content, $old_md5, $old_ver, $old_ts) = $self->_get_custom_content($class, $filename);
+    my ($old_gen, $old_md5, $old_ver, $old_ts, $old_custom)
+        = $self->_parse_generated_file($filename);
+
+    if (! $old_gen && -f $filename) {
+        croak "Cannot overwrite '$filename' without 'really_erase_my_files',"
+            . " it does not appear to have been generated by Loader"
+    }
+
+    my $custom_content = $old_custom || '';
+
+    # prepend extra custom content from a *renamed* class (singularization effect)
+    if (my $renamed_class = $self->_upgrading_classes->{$class}) {
+        my $old_filename = $self->_get_dump_filename($renamed_class);
+
+        if (-f $old_filename) {
+            my $extra_custom = ($self->_parse_generated_file ($old_filename))[4];
+
+            $extra_custom =~ s/\n\n# You can replace.*\n1;\n//;
+
+            $custom_content = join ("\n", '', $extra_custom, $custom_content)
+                if $extra_custom;
+
+            unlink $old_filename;
+        }
+    }
+
+    $custom_content ||= $self->_default_custom_content;
 
     # If upgrading to use_moose=1 replace default custom content with default Moose custom content.
     # If there is already custom content, which does not have the Moose content, add it.
     if ($self->use_moose) {
-        local $self->{use_moose} = 0;
 
-        if ($custom_content eq $self->_default_custom_content) {
-            local $self->{use_moose} = 1;
+        my $non_moose_custom_content = do {
+            local $self->{use_moose} = 0;
+            $self->_default_custom_content;
+        };
 
+        if ($custom_content eq $non_moose_custom_content) {
             $custom_content = $self->_default_custom_content;
         }
-        else {
-            local $self->{use_moose} = 1;
-
-            if ($custom_content !~ /\Q@{[$self->_default_moose_custom_content]}\E/) {
-                $custom_content .= $self->_default_custom_content;
-            }
+        elsif ($custom_content !~ /\Q@{[$self->_default_moose_custom_content]}\E/) {
+            $custom_content .= $self->_default_custom_content;
         }
     }
-
-    if (my $old_class = $self->_upgrading_classes->{$class}) {
-        my $old_filename = $self->_get_dump_filename($old_class);
-
-        my ($old_custom_content) = $self->_get_custom_content(
-            $old_class, $old_filename, 0 # do not add default comment
-        );
-
-        $old_custom_content =~ s/\n\n# You can replace.*\n1;\n//;
-
-        if ($old_custom_content) {
-            $custom_content =
-                "\n" . $old_custom_content . "\n" . $custom_content;
-        }
-
-        unlink $old_filename;
+    elsif (defined $self->use_moose && $old_gen) {
+        croak 'It is not possible to "downgrade" a schema that was loaded with use_moose => 1 to use_moose => 0, due to differing custom content'
+            if $old_gen =~ /use \s+ MooseX?\b/x;
     }
 
     $custom_content = $self->_rewrite_old_classnames($custom_content);
@@ -1335,8 +1313,6 @@ sub _write_classfile {
     my $compare_to;
     if ($old_md5) {
       $compare_to = $text . $self->_sig_comment($old_ver, $old_ts);
-      
-
       if (Digest::MD5::md5_base64($compare_to) eq $old_md5) {
         return unless $self->_upgrading_from && $is_schema;
       }
@@ -1379,48 +1355,43 @@ sub _default_custom_content {
     return $default;
 }
 
-sub _get_custom_content {
-    my ($self, $class, $filename, $add_default) = @_;
+sub _parse_generated_file {
+    my ($self, $fn) = @_;
 
-    $add_default = 1 unless defined $add_default;
+    return unless -f $fn;
 
-    return ($self->_default_custom_content) if ! -f $filename;
+    open(my $fh, '<', $fn)
+        or croak "Cannot open '$fn' for reading: $!";
 
-    open(my $fh, '<', $filename)
-        or croak "Cannot open '$filename' for reading: $!";
-
-    my $mark_re = 
+    my $mark_re =
         qr{^(# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:)([A-Za-z0-9/+]{22})\n};
 
-    my $buffer = '';
-    my ($md5, $ts, $ver);
+    my ($md5, $ts, $ver, $gen);
     while(<$fh>) {
-        if(!$md5 && /$mark_re/) {
+        if(/$mark_re/) {
+            my $pre_md5 = $1;
             $md5 = $2;
-            my $line = $1;
 
-            # Pull out the previous version and timestamp
-            ($ver, $ts) = $buffer =~ m/# Created by DBIx::Class::Schema::Loader v(.*?) @ (.*?)$/s;
+            # Pull out the version and timestamp from the line above
+            ($ver, $ts) = $gen =~ m/^# Created by DBIx::Class::Schema::Loader v(.*?) @ (.*?)\Z/m;
 
-            $buffer .= $line;
-            croak "Checksum mismatch in '$filename', the auto-generated part of the file has been modified outside of this loader.  Aborting.\nIf you want to overwrite these modifications, set the 'overwrite_modifications' loader option.\n"
-                if !$self->overwrite_modifications && Digest::MD5::md5_base64($buffer) ne $md5;
+            $gen .= $pre_md5;
+            croak "Checksum mismatch in '$fn', the auto-generated part of the file has been modified outside of this loader.  Aborting.\nIf you want to overwrite these modifications, set the 'overwrite_modifications' loader option.\n"
+                if !$self->overwrite_modifications && Digest::MD5::md5_base64($gen) ne $md5;
 
-            $buffer = '';
+            last;
         }
         else {
-            $buffer .= $_;
+            $gen .= $_;
         }
     }
 
-    croak "Cannot not overwrite '$filename' without 'really_erase_my_files',"
-        . " it does not appear to have been generated by Loader"
-            if !$md5;
+    my $custom = do { local $/; <$fh> }
+        if $md5;
 
-    # Default custom content:
-    $buffer ||= $self->_default_custom_content if $add_default;
+    close ($fh);
 
-    return ($buffer, $md5, $ver, $ts);
+    return ($gen, $md5, $ver, $ts, $custom);
 }
 
 sub _use {
